__all__ = (
    "_init_only",
    "tests_path",
)

from pathlib import Path

_init_only = {
    "eq": False,
    "repr": False,
    "match_args": False,
}

tests_path = Path(__file__).resolve().parent



__all__ = (
    "TOMLDecodeError",
    "__version__",
    "load",
    "loads",
)

from collections.abc import Callable
from typing import Any, BinaryIO

from ._toml_rs import (
    _load,
    _loads,
    _version,
)

__version__: str = _version


def load(fp: BinaryIO, /, *, parse_float: Callable[[str], Any] = float) -> dict[str, Any]:
    return _load(fp, parse_float=parse_float)


def loads(s: str, /, *, parse_float: Callable[[str], Any] = float) -> dict[str, Any]:
    if not isinstance(s, str):
        raise TypeError(f"Expected str object, not '{type(s).__name__}'")
    return _loads(s, parse_float=parse_float)


class TOMLDecodeError(ValueError):
    def __init__(
        self,
        msg: str,
        doc: str,
        pos: int,
        *args: Any,
    ):
        lineno = doc.count("\n", 0, pos) + 1
        if lineno == 1:
            colno = pos + 1
        else:
            colno = pos - doc.rindex("\n", 0, pos)

        if pos >= len(doc):
            coord_repr = "end of document"
        else:
            coord_repr = f"line {lineno}, column {colno}"
        errmsg = f"{msg} (at {coord_repr})"
        ValueError.__init__(self, errmsg)

        self.msg = msg
        self.doc = doc
        self.pos = pos
        self.lineno = lineno
        self.colno = colno



import json
from dataclasses import dataclass
from pathlib import Path

import pytest
import toml_rs as tomllib

from tests import _init_only, tests_path
from .burntsushi import (
    convert as burntsushi_convert,
    normalize as burntsushi_normalize,
)


@dataclass(**_init_only)
class MissingFile:
    path: Path


DATA_DIR = tests_path / "data"

VALID_FILES = tuple((DATA_DIR / "valid").glob("**/*.toml"))
assert VALID_FILES, "Valid TOML test files not found"

INVALID_FILES = tuple((DATA_DIR / "invalid").glob("**/*.toml"))
assert INVALID_FILES, "Invalid TOML test files not found"

_expected_files = []
for p in VALID_FILES:
    json_path = p.with_suffix(".json")
    try:
        text = json.loads(json_path.read_bytes().decode())
    except FileNotFoundError:
        text = MissingFile(json_path)
    _expected_files.append(text)
VALID_FILES_EXPECTED = tuple(_expected_files)


@pytest.mark.parametrize("invalid", INVALID_FILES, ids=lambda p: p.stem)
def test_invalid(invalid):
    toml_bytes = invalid.read_bytes()
    try:
        toml_str = toml_bytes.decode()
    except UnicodeDecodeError:
        # Some BurntSushi tests are not valid UTF-8. Skip those.
        pytest.skip(f"Invalid UTF-8: {invalid}")
    with pytest.raises(tomllib.TOMLDecodeError):
        tomllib.loads(toml_str)


VALID_PAIRS = list(zip(VALID_FILES, VALID_FILES_EXPECTED, strict=False))


@pytest.mark.parametrize(
    ("valid_file", "expected"),
    VALID_PAIRS,
    ids=[p[0].stem for p in VALID_PAIRS],
)
def test_valid(valid_file, expected):
    if isinstance(expected, MissingFile):
        # For a poor man's xfail, assert that this is one of the
        # test cases where expected data is known to be missing.
        assert valid_file.stem in {
            "qa-array-inline-nested-1000",
            "qa-table-inline-nested-1000",
        }
        pytest.xfail(f"Expected JSON missing for {valid_file.stem}")
    toml_str = valid_file.read_bytes().decode()
    actual = tomllib.loads(toml_str)
    actual = burntsushi_convert(actual)
    expected_normalized = burntsushi_normalize(expected)
    assert actual == expected_normalized



from __future__ import annotations

import sys
import timeit
from collections.abc import Callable
from pathlib import Path

import pytomlpp
import qtoml
import rtoml
import toml
import toml_rs
import tomlkit

if sys.version_info >= (3, 11):
    import tomllib
else:
    import tomli as tomllib


def benchmark(
    name: str,
    run_count: int,
    func: Callable,
    col_width: tuple,
) -> float:
    placeholder = "Running..."
    print(f"{name:>{col_width[0]}} | {placeholder}", end="", flush=True)
    time_taken = timeit.timeit(func, number=run_count)
    print("\b" * len(placeholder), end="")
    time_suffix = " s"
    print(f"{time_taken:{col_width[1] - len(time_suffix)}.3g}{time_suffix}")
    return time_taken


def run(run_count: int) -> None:
    data_path = Path(__file__).resolve().parent.parent / "tests" / "data" / "example.toml"
    test_data = data_path.read_bytes().decode()

    # qtoml has a bug making it crash without this newline normalization
    test_data = test_data.replace("\r\n", "\n")

    col_width = (10, 10, 28)
    col_head = ("parser", "exec time", "performance (more is better)")
    print(f"Parsing data.toml {run_count} times:")
    print("-" * col_width[0] + "---" + "-" * col_width[1] + "---" + col_width[2] * "-")
    print(
        f"{col_head[0]:>{col_width[0]}} | {col_head[1]:>{col_width[1]}} | {col_head[2]}"
    )
    print("-" * col_width[0] + "-+-" + "-" * col_width[1] + "-+-" + col_width[2] * "-")

    parsers = {
        "toml_rs": lambda: toml_rs.loads(test_data),
        "rtoml": lambda: rtoml.loads(test_data),
        "pytomlpp": lambda: pytomlpp.loads(test_data),
        "tomllib": lambda: tomllib.loads(test_data),
        "toml": lambda: toml.loads(test_data),
        "qtoml": lambda: qtoml.loads(test_data),
        "tomlkit": lambda: tomlkit.parse(test_data),
    }

    results = {}
    for name, func in parsers.items():
        results[name] = benchmark(name, run_count, func, col_width)
    fastest_name = min(results, key=results.get)
    fastest_time = results[fastest_name]
    print(f"\nFastest parser: {fastest_name} ({fastest_time:.5f} s)\n")

    print("Performance relative to fastest parser:")
    for name, time_taken in results.items():
        delta = fastest_time / time_taken
        print(f"{name:>{col_width[0]}} | {delta:.2%}")


if __name__ == "__main__":
    run(10_000)



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: 2021 Taneli Hukkinen
# Licensed to PSF under a Contributor Agreement.

from __future__ import annotations

from typing import Any

import pytest
import toml_rs as tomllib


def test_line_and_col():
    # invalid mantissa
    with pytest.raises(tomllib.TOMLDecodeError) as exc:
        tomllib.loads("val=.")
    msg = str(exc.value)
    assert "line 1, column 5" in msg
    assert "invalid mantissa" in msg

    # missing value
    with pytest.raises(tomllib.TOMLDecodeError) as exc:
        tomllib.loads(".")
    msg = str(exc.value)
    assert "line 1, column" in msg
    assert "missing value" in msg

    # multiple newlines
    with pytest.raises(tomllib.TOMLDecodeError) as exc:
        tomllib.loads("\n\nval=.")
    msg = str(exc.value)
    assert "line 3, column 5" in msg
    assert "invalid mantissa" in msg

    with pytest.raises(tomllib.TOMLDecodeError) as exc:
        tomllib.loads("\n\n.")
    msg = str(exc.value)
    assert "line 3, column" in msg
    assert "missing value" in msg


def test_missing_value():
    with pytest.raises(tomllib.TOMLDecodeError) as exc:
        tomllib.loads("\n\nfwfw=")
    msg = str(exc.value)
    assert "line 3, column 6" in msg
    assert "string values must be quoted" in msg


def test_invalid_char_quotes():
    with pytest.raises(tomllib.TOMLDecodeError) as exc:
        tomllib.loads("v = '\n'")
    assert "key with no value, expected `=`" in str(exc.value)


def test_type_error():
    with pytest.raises(TypeError) as exc:
        tomllib.loads(b"v = 1")  # type: ignore[arg-type]
    assert str(exc.value) in ("Expected str object, not 'bytes'", "str object expected; got bytes")

    with pytest.raises(TypeError) as exc:
        tomllib.loads(False)  # type: ignore[arg-type]  # noqa: FBT003
    assert str(exc.value) in ("Expected str object, not 'bool'", "str object expected; got bool")


def test_invalid_parse_float():
    def dict_returner(s: str) -> dict[Any, Any]:
        return {}

    def list_returner(s: str) -> list[Any]:
        return []

    err_msg = "parse_float must not return dicts or lists"

    for invalid_parse_float in (dict_returner, list_returner):
        with pytest.raises(ValueError, match=err_msg) as exc:
            tomllib.loads("f=0.1", parse_float=invalid_parse_float)
        assert str(exc.value) == err_msg


def test_tomldecodeerror_attributes():
    msg = "error parsing"
    doc = "v=1\n[table]\nv='val'"
    pos = 13
    formatted_msg = "error parsing (at line 3, column 2)"

    e = tomllib.TOMLDecodeError(msg, doc, pos)
    assert e.args == (formatted_msg,)
    assert str(e) == formatted_msg
    assert e.msg == msg
    assert e.doc == doc
    assert e.pos == pos
    assert e.lineno == 3
    assert e.colno == 2



from pathlib import Path

import pytest
import toml_rs
import tomllib

from tests import tests_path

TOML = tests_path / "data" / "example.toml"


def test_toml():
    toml_str = TOML.read_text(encoding="utf-8")
    assert tomllib.loads(toml_str) == toml_rs.loads(toml_str)


def test_toml_text_mode_typeerror():
    with Path(TOML).open(encoding="utf-8") as f, pytest.raises(TypeError) as exc1:
        tomllib.load(f)
    tomllib_err = str(exc1.value)

    with Path(TOML).open(encoding="utf-8") as f, pytest.raises(TypeError) as exc2:
        toml_rs.load(f)
    toml_rs_err = str(exc2.value)

    err_msg = "File must be opened in binary mode, e.g. use `open('foo.toml', 'rb')`"
    assert err_msg in tomllib_err
    assert err_msg in toml_rs_err
    assert tomllib_err == toml_rs_err
